@@ -1,11 +1,15 @@
 #! /usr/bin/env python
 # Last Change: Mon Aug 20 08:00 PM 2007 J
+from __future__ import division, print_function, absolute_import
+
 import re
 import itertools
+import datetime
+from functools import partial
 
 import numpy as np
 
-from scipy.io.arff.utils import partial
+from scipy._lib.six import next
 
 """A module to read arff files."""
 
@@ -39,7 +43,7 @@ r_attribute = re.compile(r'^@[Aa][Tt][Tt][Rr][Ii][Bb][Uu][Tt][Ee]\s*(..*$)')
 
 # To get attributes name enclosed with ''
 r_comattrval = re.compile(r"'(..+)'\s+(..+$)")
-# To get attributes name enclosed with '', possibly spread accross multilines
+# To get attributes name enclosed with '', possibly spread across multilines
 r_mcomattrval = re.compile(r"'([..\n]+)'\s+(..+$)")
 # To get normal attributes
 r_wcomattrval = re.compile(r"(\S+)\s+(..+$)")
@@ -47,9 +51,12 @@ r_wcomattrval = re.compile(r"(\S+)\s+(..+$)")
 #-------------------------
 # Module defined exception
 #-------------------------
+
+
 class ArffError(IOError):
     pass
 
+
 class ParseArffError(ArffError):
     pass
 
@@ -58,6 +65,8 @@ class ParseArffError(ArffError):
 #------------------
 
 # An attribute  is defined as @attribute name value
+
+
 def parse_type(attrtype):
     """Given an arff attribute value (meta data), returns its type.
 
@@ -75,6 +84,8 @@ def parse_type(attrtype):
         return 'string'
     elif uattribute[:len('relational')] == 'relational':
         return 'relational'
+    elif uattribute[:len('date')] == 'date':
+        return 'date'
     else:
         raise ParseArffError("unknown attribute %s" % uattribute)
 
@@ -83,17 +94,19 @@ def get_nominal(attribute):
     """If attribute is nominal, returns a list of the values"""
     return attribute.split(',')
 
+
 def read_data_list(ofile):
     """Read each line of the iterable and put it in a list."""
-    data = [ofile.next()]
+    data = [next(ofile)]
     if data[0].strip()[0] == '{':
         raise ValueError("This looks like a sparse ARFF: not supported yet")
     data.extend([i for i in ofile])
     return data
 
+
 def get_ndata(ofile):
     """Read the whole file to get number of data attributes."""
-    data = [ofile.next()]
+    data = [next(ofile)]
     loc = 1
     if data[0].strip()[0] == '{':
         raise ValueError("This looks like a sparse ARFF: not supported yet")
@@ -101,37 +114,109 @@ def get_ndata(ofile):
         loc += 1
     return loc
 
+
 def maxnomlen(atrv):
-    """Given a string contening a nominal type definition, returns the string
-    len of the biggest component.
+    """Given a string containing a nominal type definition, returns the
+    string len of the biggest component.
 
     A nominal type is defined as seomthing framed between brace ({}).
 
-    Example: maxnomlen("{floup, bouga, fl, ratata}") returns 6 (the size of
-    ratata, the longest nominal value)."""
+    Parameters
+    ----------
+    atrv : str
+       Nominal type definition
+
+    Returns
+    -------
+    slen : int
+       length of longest component
+
+    Examples
+    --------
+    maxnomlen("{floup, bouga, fl, ratata}") returns 6 (the size of
+    ratata, the longest nominal value).
+
+    >>> maxnomlen("{floup, bouga, fl, ratata}")
+    6
+    """
     nomtp = get_nom_val(atrv)
     return max(len(i) for i in nomtp)
 
+
 def get_nom_val(atrv):
-    """Given a string contening a nominal type, returns a tuple of the possible
-    values.
+    """Given a string containing a nominal type, returns a tuple of the
+    possible values.
 
-    A nominal type is defined as something framed between brace ({}).
+    A nominal type is defined as something framed between braces ({}).
 
-    Example: get_nom_val("{floup, bouga, fl, ratata}") returns ("floup",
-    "bouga", "fl", "ratata")."""
-    r_nominal = re.compile('{(..+)}')
+    Parameters
+    ----------
+    atrv : str
+       Nominal type definition
+
+    Returns
+    -------
+    poss_vals : tuple
+       possible values
+
+    Examples
+    --------
+    >>> get_nom_val("{floup, bouga, fl, ratata}")
+    ('floup', 'bouga', 'fl', 'ratata')
+    """
+    r_nominal = re.compile('{(.+)}')
     m = r_nominal.match(atrv)
     if m:
         return tuple(i.strip() for i in m.group(1).split(','))
     else:
         raise ValueError("This does not look like a nominal string")
 
+
+def get_date_format(atrv):
+    r_date = re.compile(r"[Dd][Aa][Tt][Ee]\s+[\"']?(.+?)[\"']?$")
+    m = r_date.match(atrv)
+    if m:
+        pattern = m.group(1).strip()
+        # convert time pattern from Java's SimpleDateFormat to C's format
+        datetime_unit = None
+        if "yyyy" in pattern:
+            pattern = pattern.replace("yyyy", "%Y")
+            datetime_unit = "Y"
+        elif "yy":
+            pattern = pattern.replace("yy", "%y")
+            datetime_unit = "Y"
+        if "MM" in pattern:
+            pattern = pattern.replace("MM", "%m")
+            datetime_unit = "M"
+        if "dd" in pattern:
+            pattern = pattern.replace("dd", "%d")
+            datetime_unit = "D"
+        if "HH" in pattern:
+            pattern = pattern.replace("HH", "%H")
+            datetime_unit = "h"
+        if "mm" in pattern:
+            pattern = pattern.replace("mm", "%M")
+            datetime_unit = "m"
+        if "ss" in pattern:
+            pattern = pattern.replace("ss", "%S")
+            datetime_unit = "s"
+        if "z" in pattern or "Z" in pattern:
+            raise ValueError("Date type attributes with time zone not supported, yet")
+
+        if datetime_unit is None:
+            raise ValueError("Invalid or unsupported date format")
+
+        return pattern, datetime_unit
+    else:
+        raise ValueError("Invalid or no date format")
+
+
 def go_data(ofile):
     """Skip header.
 
     the first next() call of the returned iterator will be the @data line"""
-    return itertools.dropwhile(lambda x : not r_datameta.match(x), ofile)
+    return itertools.dropwhile(lambda x: not r_datameta.match(x), ofile)
+
 
 #----------------
 # Parsing header
@@ -141,28 +226,42 @@ def tokenize_attribute(iterable, attribute):
 
     Given a raw string attribute, try to get the name and type of the
     attribute. Constraints:
-        - The first line must start with @attribute (case insensitive, and
-          space like characters begore @attribute are allowed)
-        - Works also if the attribute is spread on multilines.
-        - Works if empty lines or comments are in between
 
-    :Parameters:
-        attribute : str
-            the attribute string.
+    * The first line must start with @attribute (case insensitive, and
+      space like characters before @attribute are allowed)
+    * Works also if the attribute is spread on multilines.
+    * Works if empty lines or comments are in between
 
-    :Returns:
-        name : str
-            name of the attribute
-        value : str
-            value of the attribute
-        next : str
-            next line to be parsed
+    Parameters
+    ----------
+    attribute : str
+       the attribute string.
 
-    Example:
-        - if attribute is a string defined in python as r"floupi real", will
-          return floupi as name, and real as value.
-        - if attribute is r"'floupi 2' real", will return 'floupi 2' as name,
-          and real as value. """
+    Returns
+    -------
+    name : str
+       name of the attribute
+    value : str
+       value of the attribute
+    next : str
+       next line to be parsed
+
+    Examples
+    --------
+    If attribute is a string defined in python as r"floupi real", will
+    return floupi as name, and real as value.
+
+    >>> iterable = iter([0] * 10) # dummy iterator
+    >>> tokenize_attribute(iterable, r"@attribute floupi real")
+    ('floupi', 'real', 0)
+
+    If attribute is r"'floupi 2' real", will return 'floupi 2' as name,
+    and real as value.
+
+    >>> tokenize_attribute(iterable, r"  @attribute 'floupi 2' real   ")
+    ('floupi 2', 'real', 0)
+
+    """
     sattr = attribute.strip()
     mattr = r_attribute.match(sattr)
     if mattr:
@@ -170,40 +269,22 @@ def tokenize_attribute(iterable, attribute):
         atrv = mattr.group(1)
         if r_comattrval.match(atrv):
             name, type = tokenize_single_comma(atrv)
-            next = iterable.next()
+            next_item = next(iterable)
         elif r_wcomattrval.match(atrv):
             name, type = tokenize_single_wcomma(atrv)
-            next = iterable.next()
+            next_item = next(iterable)
         else:
             # Not sure we should support this, as it does not seem supported by
             # weka.
             raise ValueError("multi line not supported yet")
-            #name, type, next = tokenize_multilines(iterable, atrv)
+            #name, type, next_item = tokenize_multilines(iterable, atrv)
     else:
         raise ValueError("First line unparsable: %s" % sattr)
 
     if type == 'relational':
         raise ValueError("relational attributes not supported yet")
-    return name, type, next
+    return name, type, next_item
 
-def tokenize_multilines(iterable, val):
-    """Can tokenize an attribute spread over several lines."""
-    # If one line does not match, read all the following lines up to next
-    # line with meta character, and try to parse everything up to there.
-    if not r_mcomattrval.match(val):
-        all = [val]
-        i = iterable.next()
-        while not r_meta.match(i):
-            all.append(i)
-            i = iterable.next()
-        if r_mend.search(i):
-            raise ValueError("relational attribute not supported yet")
-        print "".join(all[:-1])
-        m = r_comattrval.match("".join(all[:-1]))
-        return m.group(1), m.group(2), i
-    else:
-        raise ValueError("Cannot parse attribute names spread over multi "\
-                        "lines yet")
 
 def tokenize_single_comma(val):
     # XXX we match twice the same string (here and at the caller level). It is
@@ -219,6 +300,7 @@ def tokenize_single_comma(val):
         raise ValueError("Error while tokenizing single %s" % val)
     return name, type
 
+
 def tokenize_single_wcomma(val):
     # XXX we match twice the same string (here and at the caller level). It is
     # stupid, but it is easier for now...
@@ -233,13 +315,14 @@ def tokenize_single_wcomma(val):
         raise ValueError("Error while tokenizing single %s" % val)
     return name, type
 
+
 def read_header(ofile):
     """Read the header of the iterable ofile."""
-    i = ofile.next()
+    i = next(ofile)
 
     # Pass first comments
     while r_comment.match(i):
-        i = ofile.next()
+        i = next(ofile)
 
     # Header is everything up to DATA attribute ?
     relation = None
@@ -257,23 +340,45 @@ def read_header(ofile):
                     relation = isrel.group(1)
                 else:
                     raise ValueError("Error parsing line %s" % i)
-                i = ofile.next()
+                i = next(ofile)
         else:
-            i = ofile.next()
+            i = next(ofile)
 
     return relation, attributes
 
+
 #--------------------
 # Parsing actual data
 #--------------------
 def safe_float(x):
     """given a string x, convert it to a float. If the stripped string is a ?,
-    return a Nan (missing value)."""
-    if x.strip() == '?':
+    return a Nan (missing value).
+
+    Parameters
+    ----------
+    x : str
+       string to convert
+
+    Returns
+    -------
+    f : float
+       where float can be nan
+
+    Examples
+    --------
+    >>> safe_float('1')
+    1.0
+    >>> safe_float('1\\n')
+    1.0
+    >>> safe_float('?\\n')
+    nan
+    """
+    if '?' in x:
         return np.nan
     else:
         return np.float(x)
 
+
 def safe_nominal(value, pvalue):
     svalue = value.strip()
     if svalue in pvalue:
@@ -283,41 +388,70 @@ def safe_nominal(value, pvalue):
     else:
         raise ValueError("%s value not in %s" % (str(svalue), str(pvalue)))
 
+
+def safe_date(value, date_format, datetime_unit):
+    date_str = value.strip().strip("'").strip('"')
+    if date_str == '?':
+        return np.datetime64('NaT', datetime_unit)
+    else:
+        dt = datetime.datetime.strptime(date_str, date_format)
+        return np.datetime64(dt).astype("datetime64[%s]" % datetime_unit)
+
+
 def get_delim(line):
     """Given a string representing a line of data, check whether the
-    delimiter is ',' or space."""
-    l = line.split(',')
-    if len(l) > 1:
-        return ','
-    else:
-        l = line.split(' ')
-        if len(l) > 1:
-            return ' '
-        else:
-            raise ValueError("delimiter not understood: " + line)
+    delimiter is ',' or space.
 
-class MetaData:
+    Parameters
+    ----------
+    line : str
+       line of data
+
+    Returns
+    -------
+    delim : {',', ' '}
+
+    Examples
+    --------
+    >>> get_delim(',')
+    ','
+    >>> get_delim(' ')
+    ' '
+    >>> get_delim(', ')
+    ','
+    >>> get_delim('x')
+    Traceback (most recent call last):
+       ...
+    ValueError: delimiter not understood: x
+    """
+    if ',' in line:
+        return ','
+    if ' ' in line:
+        return ' '
+    raise ValueError("delimiter not understood: " + line)
+
+
+class MetaData(object):
     """Small container to keep useful informations on a ARFF dataset.
 
     Knows about attributes names and types.
 
-    :Example:
-
-        data, meta = loadarff('iris.arff')
-        # This will print the attributes names of the iris.arff dataset
-        for i in meta:
-            print i
-        # This works too
-        meta.names()
-        # Getting attribute type
-        types = meta.types()
-
-    :Note:
-
-        Also maintains the list of attributes in order, i.e. doing for i in
-        meta, where meta is an instance of MetaData, will return the different
-        attribute names in the order they were defined.
+    Examples
+    --------
+    data, meta = loadarff('iris.arff')
+    # This will print the attributes names of the iris.arff dataset
+    for i in meta:
+        print i
+    # This works too
+    meta.names()
+    # Getting attribute type
+    types = meta.types()
 
+    Notes
+    -----
+    Also maintains the list of attributes in order, i.e. doing for i in
+    meta, where meta is an instance of MetaData, will return the
+    different attribute names in the order they were defined.
     """
     def __init__(self, rel, attr):
         self.name = rel
@@ -330,6 +464,8 @@ class MetaData:
             self._attrnames.append(name)
             if tp == 'nominal':
                 self._attributes[name] = (tp, get_nom_val(value))
+            elif tp == 'date':
+                self._attributes[name] = (tp, get_date_format(value)[0])
             else:
                 self._attributes[name] = (tp, None)
 
@@ -355,40 +491,71 @@ class MetaData:
 
     def types(self):
         """Return the list of attribute types."""
-        return [v[0] for v in self._attributes.values()]
+        attr_types = [self._attributes[name][0] for name in self._attrnames]
+        return attr_types
 
-def loadarff(filename):
-    """Read an arff file.
 
-    :Args:
-
-        filename: str
-            the name of the file
-
-    :Returns:
-
-        data: record array
-            the data of the arff file. Each record corresponds to one attribute.
-        meta: MetaData
-            this contains informations about the arff file, like type and names
-            of attributes, the relation (name of the dataset), etc...
-
-    :Note:
-
-        This function should be able to read most arff files. Not implemented
-        functionalities include:
-            - date type attributes
-            - string type attributes
-
-        It can read files with numeric and nominal attributes.
-        It can read files with sparse data (? in the file).
+def loadarff(f):
     """
-    ofile = open(filename)
+    Read an arff file.
 
+    The data is returned as a record array, which can be accessed much like
+    a dictionary of numpy arrays.  For example, if one of the attributes is
+    called 'pressure', then its first 10 data points can be accessed from the
+    ``data`` record array like so: ``data['pressure'][0:10]``
+
+
+    Parameters
+    ----------
+    f : file-like or str
+       File-like object to read from, or filename to open.
+
+    Returns
+    -------
+    data : record array
+       The data of the arff file, accessible by attribute names.
+    meta : `MetaData`
+       Contains information about the arff file such as name and
+       type of attributes, the relation (name of the dataset), etc...
+
+    Raises
+    ------
+    ParseArffError
+        This is raised if the given file is not ARFF-formatted.
+    NotImplementedError
+        The ARFF file has an attribute which is not supported yet.
+
+    Notes
+    -----
+
+    This function should be able to read most arff files. Not
+    implemented functionality include:
+
+    * date type attributes
+    * string type attributes
+
+    It can read files with numeric and nominal attributes.  It cannot read
+    files with sparse data ({} in the file).  However, this function can
+    read files with missing data (? in the file), representing the data
+    points as NaNs.
+
+    """
+    if hasattr(f, 'read'):
+        ofile = f
+    else:
+        ofile = open(f, 'rt')
+    try:
+        return _loadarff(ofile)
+    finally:
+        if ofile is not f:  # only close what we opened
+            ofile.close()
+
+
+def _loadarff(ofile):
     # Parse the header file
     try:
         rel, attr = read_header(ofile)
-    except ValueError, e:
+    except ValueError as e:
         msg = "Error while parsing header, error was: " + str(e)
         raise ParseArffError(msg)
 
@@ -408,20 +575,22 @@ def loadarff(filename):
 
     # This can be used once we want to support integer as integer values and
     # not as numeric anymore (using masked arrays ?).
-    acls2dtype = {'real' : np.float, 'integer' : np.float, 'numeric' : np.float}
-    acls2conv = {'real' : safe_float, 'integer' : safe_float, 'numeric' : safe_float}
+    acls2dtype = {'real': np.float, 'integer': np.float, 'numeric': np.float}
+    acls2conv = {'real': safe_float, 'integer': safe_float, 'numeric': safe_float}
     descr = []
     convertors = []
     if not hasstr:
         for name, value in attr:
             type = parse_type(value)
             if type == 'date':
-                raise ValueError("date type not supported yet, sorry")
+                date_format, datetime_unit = get_date_format(value)
+                descr.append((name, "datetime64[%s]" % datetime_unit))
+                convertors.append(partial(safe_date, date_format=date_format, datetime_unit=datetime_unit))
             elif type == 'nominal':
                 n = maxnomlen(value)
                 descr.append((name, 'S%d' % n))
                 pvalue = get_nom_val(value)
-                convertors.append(partial(safe_nominal, pvalue = pvalue))
+                convertors.append(partial(safe_nominal, pvalue=pvalue))
             else:
                 descr.append((name, acls2dtype[type]))
                 convertors.append(safe_float)
@@ -437,26 +606,24 @@ def loadarff(filename):
     # Get the delimiter from the first line of data:
     def next_data_line(row_iter):
         """Assumes we are already in the data part (eg after @data)."""
-        raw = row_iter.next()
-        while r_empty.match(raw):
-            raw = row_iter.next()
-        while r_comment.match(raw):
-            raw = row_iter.next()
+        raw = next(row_iter)
+        while r_empty.match(raw) or r_comment.match(raw):
+            raw = next(row_iter)
         return raw
 
     try:
         try:
             dtline = next_data_line(ofile)
             delim = get_delim(dtline)
-        except ValueError, e:
+        except ValueError as e:
             raise ParseArffError("Error while parsing delimiter: " + str(e))
     finally:
         ofile.seek(0, 0)
         ofile = go_data(ofile)
         # skip the @data line
-        ofile.next()
+        next(ofile)
 
-    def generator(row_iter, delim = ','):
+    def generator(row_iter, delim=','):
         # TODO: this is where we are spending times (~80%). I think things
         # could be made more efficiently:
         #   - We could for example "compile" the function, because some values
@@ -470,27 +637,29 @@ def loadarff(filename):
 
         # We do not abstract skipping comments and empty lines for performances
         # reason.
-        raw = row_iter.next()
-        while r_empty.match(raw):
-            raw = row_iter.next()
-        while r_comment.match(raw):
-            raw = row_iter.next()
+        raw = next(row_iter)
+        while r_empty.match(raw) or r_comment.match(raw):
+            raw = next(row_iter)
+
+        # 'compiling' the range since it does not change
+        # Note, I have already tried zipping the converters and
+        # row elements and got slightly worse performance.
+        elems = list(range(ni))
 
         row = raw.split(delim)
-        yield tuple([convertors[i](row[i]) for i in range(ni)])
+        yield tuple([convertors[i](row[i]) for i in elems])
         for raw in row_iter:
-            while r_comment.match(raw):
-                raw = row_iter.next()
-            while r_empty.match(raw):
-                raw = row_iter.next()
+            while r_comment.match(raw) or r_empty.match(raw):
+                raw = next(row_iter)
             row = raw.split(delim)
-            yield tuple([convertors[i](row[i]) for i in range(ni)])
+            yield tuple([convertors[i](row[i]) for i in elems])
 
-    a = generator(ofile, delim = delim)
+    a = generator(ofile, delim=delim)
     # No error should happen here: it is a bug otherwise
     data = np.fromiter(a, descr)
     return data, meta
 
+
 #-----
 # Misc
 #-----
@@ -498,74 +667,33 @@ def basic_stats(data):
     nbfac = data.size * 1. / (data.size - 1)
     return np.nanmin(data), np.nanmax(data), np.mean(data), np.std(data) * nbfac
 
+
 def print_attribute(name, tp, data):
     type = tp[0]
     if type == 'numeric' or type == 'real' or type == 'integer':
         min, max, mean, std = basic_stats(data)
-        print "%s,%s,%f,%f,%f,%f" % (name, type, min, max, mean, std)
+        print("%s,%s,%f,%f,%f,%f" % (name, type, min, max, mean, std))
     else:
         msg = name + ",{"
         for i in range(len(tp[1])-1):
             msg += tp[1][i] + ","
         msg += tp[1][-1]
         msg += "}"
-        print msg
+        print(msg)
+
 
 def test_weka(filename):
     data, meta = loadarff(filename)
-    print len(data.dtype)
-    print data.size
+    print(len(data.dtype))
+    print(data.size)
     for i in meta:
         print_attribute(i,meta[i],data[i])
 
-def floupi(filename):
-    data, meta = loadarff(filename)
-    from attrselect import print_dataset_info
-    print_dataset_info(data)
-    print "relation %s, has %d instances" % (meta.name, data.size)
-    itp = iter(types)
-    for i in data.dtype.names:
-        print_attribute(i,itp.next(),data[i])
-        #tp = itp.next()
-        #if tp == 'numeric' or tp == 'real' or tp == 'integer':
-        #    min, max, mean, std = basic_stats(data[i])
-        #    print "\tinstance %s: min %f, max %f, mean %f, std %f" % \
-        #            (i, min, max, mean, std)
-        #else:
-        #    print "\tinstance %s is non numeric" % i
+# make sure nose does not find this as a test
+test_weka.__test__ = False
+
 
 if __name__ == '__main__':
-    #import glob
-    #for i in glob.glob('arff.bak/data/*'):
-    #    relation, attributes = read_header(open(i))
-    #    print "Parsing header of %s: relation %s, %d attributes" % (i,
-    #            relation, len(attributes))
-
     import sys
     filename = sys.argv[1]
-    #filename = 'arff.bak/data/pharynx.arff'
-    #floupi(filename)
     test_weka(filename)
-
-    #gf = []
-    #wf = []
-    #for i in glob.glob('arff.bak/data/*'):
-    #    try:
-    #        print "=============== reading %s ======================" % i
-    #        floupi(i)
-    #        gf.append(i)
-    #    except ValueError, e:
-    #        print "!!!! Error parsing the file !!!!!"
-    #        print e
-    #        wf.append(i)
-    #    except IndexError, e:
-    #        print "!!!! Error parsing the file !!!!!"
-    #        print e
-    #        wf.append(i)
-    #    except ArffError, e:
-    #        print "!!!! Error parsing the file !!!!!"
-    #        print e
-    #        wf.append(i)
-
-    #print "%d good files" % len(gf)
-    #print "%d bad files" % len(wf)
