@@ -1,11 +1,14 @@
+from __future__ import division, print_function, absolute_import
 
 import math
 import numpy as np
-from scipy.misc import comb
+from scipy._lib.six import xrange
+from scipy._lib.six import string_types
+
 
 __all__ = ['tri', 'tril', 'triu', 'toeplitz', 'circulant', 'hankel',
-           'hadamard', 'leslie', 'all_mat', 'kron', 'block_diag', 'companion',
-           'hilbert', 'invhilbert']
+           'hadamard', 'leslie', 'kron', 'block_diag', 'companion',
+           'helmert', 'hilbert', 'invhilbert', 'pascal', 'invpascal', 'dft']
 
 
 #-----------------------------------------------------------------------------
@@ -25,21 +28,22 @@ def tri(N, M=None, k=0, dtype=None):
 
     Parameters
     ----------
-    N : integer
+    N : int
         The size of the first dimension of the matrix.
-    M : integer or None
+    M : int or None, optional
         The size of the second dimension of the matrix. If `M` is None,
         `M = N` is assumed.
-    k : integer
+    k : int, optional
         Number of subdiagonal below which matrix is filled with ones.
         `k` = 0 is the main diagonal, `k` < 0 subdiagonal and `k` > 0
         superdiagonal.
-    dtype : dtype
+    dtype : dtype, optional
         Data type of the matrix.
 
     Returns
     -------
-    A : array, shape (N, M)
+    tri : (N, M) ndarray
+        Tri matrix.
 
     Examples
     --------
@@ -54,32 +58,37 @@ def tri(N, M=None, k=0, dtype=None):
            [1, 1, 0, 0, 0]])
 
     """
-    if M is None: M = N
-    if type(M) == type('d'):
+    if M is None:
+        M = N
+    if isinstance(M, string_types):
         #pearu: any objections to remove this feature?
         #       As tri(N,'d') is equivalent to tri(N,dtype='d')
         dtype = M
         M = N
-    m = np.greater_equal(np.subtract.outer(np.arange(N), np.arange(M)),-k)
+    m = np.greater_equal(np.subtract.outer(np.arange(N), np.arange(M)), -k)
     if dtype is None:
         return m
     else:
         return m.astype(dtype)
 
+
 def tril(m, k=0):
-    """Construct a copy of a matrix with elements above the k-th diagonal zeroed.
+    """
+    Make a copy of a matrix with elements above the k-th diagonal zeroed.
 
     Parameters
     ----------
-    m : array
+    m : array_like
         Matrix whose elements to return
-    k : integer
+    k : int, optional
         Diagonal above which to zero elements.
-        k == 0 is the main diagonal, k < 0 subdiagonal and k > 0 superdiagonal.
+        `k` == 0 is the main diagonal, `k` < 0 subdiagonal and
+        `k` > 0 superdiagonal.
 
     Returns
     -------
-    A : array, shape m.shape, dtype m.dtype
+    tril : ndarray
+        Return is the same shape and type as `m`.
 
     Examples
     --------
@@ -92,27 +101,32 @@ def tril(m, k=0):
 
     """
     m = np.asarray(m)
-    out = tri(m.shape[0], m.shape[1], k=k, dtype=m.dtype.char)*m
+    out = tri(m.shape[0], m.shape[1], k=k, dtype=m.dtype.char) * m
     return out
 
+
 def triu(m, k=0):
-    """Construct a copy of a matrix with elements below the k-th diagonal zeroed.
+    """
+    Make a copy of a matrix with elements below the k-th diagonal zeroed.
 
     Parameters
     ----------
-    m : array
+    m : array_like
         Matrix whose elements to return
-    k : integer
+    k : int, optional
         Diagonal below which to zero elements.
-        k == 0 is the main diagonal, k < 0 subdiagonal and k > 0 superdiagonal.
+        `k` == 0 is the main diagonal, `k` < 0 subdiagonal and
+        `k` > 0 superdiagonal.
 
     Returns
     -------
-    A : array, shape m.shape, dtype m.dtype
+    triu : ndarray
+        Return matrix with zeroed elements below the k-th diagonal and has
+        same shape and type as `m`.
 
     Examples
     --------
-    >>> from scipy.linalg import tril
+    >>> from scipy.linalg import triu
     >>> triu([[1,2,3],[4,5,6],[7,8,9],[10,11,12]], -1)
     array([[ 1,  2,  3],
            [ 4,  5,  6],
@@ -121,7 +135,7 @@ def triu(m, k=0):
 
     """
     m = np.asarray(m)
-    out = (1-tri(m.shape[0], m.shape[1], k-1, m.dtype.char))*m
+    out = (1 - tri(m.shape[0], m.shape[1], k - 1, m.dtype.char)) * m
     return out
 
 
@@ -138,7 +152,7 @@ def toeplitz(c, r=None):
     c : array_like
         First column of the matrix.  Whatever the actual shape of `c`, it
         will be converted to a 1-D array.
-    r : array_like
+    r : array_like, optional
         First row of the matrix. If None, ``r = conjugate(c)`` is assumed;
         in this case, if c[0] is real, the result is a Hermitian matrix.
         r[0] is ignored; the first row of the returned matrix is
@@ -147,7 +161,7 @@ def toeplitz(c, r=None):
 
     Returns
     -------
-    A : array, shape (len(c), len(r))
+    A : (len(c), len(r)) ndarray
         The Toeplitz matrix. Dtype is the same as ``(c[0] + r[0]).dtype``.
 
     See also
@@ -182,24 +196,25 @@ def toeplitz(c, r=None):
     # Form a 1D array of values to be used in the matrix, containing a reversed
     # copy of r[1:], followed by c.
     vals = np.concatenate((r[-1:0:-1], c))
-    a, b = np.ogrid[0:len(c), len(r)-1:-1:-1]
+    a, b = np.ogrid[0:len(c), len(r) - 1:-1:-1]
     indx = a + b
-    # `indx` is a 2D array of indices into the 1D array `vals`, arranged so that
-    # `vals[indx]` is the Toeplitz matrix.
+    # `indx` is a 2D array of indices into the 1D array `vals`, arranged so
+    # that `vals[indx]` is the Toeplitz matrix.
     return vals[indx]
 
+
 def circulant(c):
     """
     Construct a circulant matrix.
 
     Parameters
     ----------
-    c : array_like
+    c : (N,) array_like
         1-D array, the first column of the matrix.
 
     Returns
     -------
-    A : array, shape (len(c), len(c))
+    A : (N, N) ndarray
         A circulant matrix whose first column is `c`.
 
     See also
@@ -227,6 +242,7 @@ def circulant(c):
     # the circulant matrix.
     return c[indx]
 
+
 def hankel(c, r=None):
     """
     Construct a Hankel matrix.
@@ -240,7 +256,7 @@ def hankel(c, r=None):
     c : array_like
         First column of the matrix.  Whatever the actual shape of `c`, it
         will be converted to a 1-D array.
-    r : array_like, 1D
+    r : array_like, optional
         Last row of the matrix. If None, ``r = zeros_like(c)`` is assumed.
         r[0] is ignored; the last row of the returned matrix is
         ``[c[-1], r[1:]]``.  Whatever the actual shape of `r`, it will be
@@ -248,7 +264,7 @@ def hankel(c, r=None):
 
     Returns
     -------
-    A : array, shape (len(c), len(r))
+    A : (len(c), len(r)) ndarray
         The Hankel matrix. Dtype is the same as ``(c[0] + r[0]).dtype``.
 
     See also
@@ -280,27 +296,28 @@ def hankel(c, r=None):
     vals = np.concatenate((c, r[1:]))
     a, b = np.ogrid[0:len(c), 0:len(r)]
     indx = a + b
-    # `indx` is a 2D array of indices into the 1D array `vals`, arranged so that
-    # `vals[indx]` is the Hankel matrix.
+    # `indx` is a 2D array of indices into the 1D array `vals`, arranged so
+    # that `vals[indx]` is the Hankel matrix.
     return vals[indx]
 
+
 def hadamard(n, dtype=int):
     """
     Construct a Hadamard matrix.
 
-    `hadamard(n)` constructs an n-by-n Hadamard matrix, using Sylvester's
+    Constructs an n-by-n Hadamard matrix, using Sylvester's
     construction.  `n` must be a power of 2.
 
     Parameters
     ----------
     n : int
         The order of the matrix.  `n` must be a power of 2.
-    dtype : numpy dtype
+    dtype : dtype, optional
         The data type of the array to be constructed.
 
     Returns
     -------
-    H : ndarray with shape (n, n)
+    H : (n, n) ndarray
         The Hadamard matrix.
 
     Notes
@@ -309,6 +326,7 @@ def hadamard(n, dtype=int):
 
     Examples
     --------
+    >>> from scipy.linalg import hadamard
     >>> hadamard(2, dtype=complex)
     array([[ 1.+0.j,  1.+0.j],
            [ 1.+0.j, -1.-0.j]])
@@ -328,7 +346,8 @@ def hadamard(n, dtype=int):
     else:
         lg2 = int(math.log(n, 2))
     if 2 ** lg2 != n:
-        raise ValueError("n must be an positive integer, and n must be power of 2")
+        raise ValueError("n must be an positive integer, and n must be "
+                         "a power of 2")
 
     H = np.array([[1]], dtype=dtype)
 
@@ -348,17 +367,16 @@ def leslie(f, s):
 
     Parameters
     ----------
-    f : array_like
-        The "fecundity" coefficients, has to be 1-D.
-    s : array_like
+    f : (N,) array_like
+        The "fecundity" coefficients.
+    s : (N-1,) array_like
         The "survival" coefficients, has to be 1-D.  The length of `s`
         must be one less than the length of `f`, and it must be at least 1.
 
     Returns
     -------
-    L : ndarray
-        Returns a 2-D ndarray of shape ``(n, n)``, where `n` is the
-        length of `f`.  The array is zero except for the first row,
+    L : (N, N) ndarray
+        The array is zero except for the first row,
         which is `f`, and the first sub-diagonal, which is `s`.
         The data-type of the array will be the data-type of ``f[0]+s[0]``.
 
@@ -383,6 +401,7 @@ def leslie(f, s):
 
     Examples
     --------
+    >>> from scipy.linalg import leslie
     >>> leslie([0.1, 2.0, 1.0, 0.1], [0.2, 0.8, 0.7])
     array([[ 0.1,  2. ,  1. ,  0.1],
            [ 0.2,  0. ,  0. ,  0. ],
@@ -404,17 +423,15 @@ def leslie(f, s):
 
     tmp = f[0] + s[0]
     n = f.size
-    a = np.zeros((n,n), dtype=tmp.dtype)
+    a = np.zeros((n, n), dtype=tmp.dtype)
     a[0] = f
-    a[range(1,n), range(0,n-1)] = s
+    a[list(range(1, n)), list(range(0, n - 1))] = s
     return a
 
 
-def all_mat(*args):
-    return map(np.matrix,args)
-
-def kron(a,b):
-    """Kronecker product of a and b.
+def kron(a, b):
+    """
+    Kronecker product.
 
     The result is the block matrix::
 
@@ -425,17 +442,20 @@ def kron(a,b):
 
     Parameters
     ----------
-    a : array, shape (M, N)
-    b : array, shape (P, Q)
+    a : (M, N) ndarray
+        Input array
+    b : (P, Q) ndarray
+        Input array
 
     Returns
     -------
-    A : array, shape (M*P, N*Q)
-        Kronecker product of a and b
+    A : (M*P, N*Q) ndarray
+        Kronecker product of `a` and `b`.
 
     Examples
     --------
-    >>> from scipy import kron, array
+    >>> from numpy import array
+    >>> from scipy.linalg import kron
     >>> kron(array([[1,2],[3,4]]), array([[1,1,1]]))
     array([[1, 1, 1, 2, 2, 2],
            [3, 3, 3, 4, 4, 4]])
@@ -445,10 +465,11 @@ def kron(a,b):
         a = np.reshape(a, a.shape)
     if not b.flags['CONTIGUOUS']:
         b = np.reshape(b, b.shape)
-    o = np.outer(a,b)
+    o = np.outer(a, b)
     o = o.reshape(a.shape + b.shape)
     return np.concatenate(np.concatenate(o, axis=1), axis=1)
 
+
 def block_diag(*arrs):
     """
     Create a block diagonal matrix from provided arrays.
@@ -463,7 +484,7 @@ def block_diag(*arrs):
     Parameters
     ----------
     A, B, C, ... : array_like, up to 2-D
-        Input arrays.  A 1-D array or array_like sequence of length `n`is
+        Input arrays.  A 1-D array or array_like sequence of length `n` is
         treated as a 2-D array with shape ``(1,n)``.
 
     Returns
@@ -477,8 +498,11 @@ def block_diag(*arrs):
     If all the input arrays are square, the output is known as a
     block diagonal matrix.
 
+    Empty sequences (i.e., array-likes of zero size) are ignored.
+
     Examples
     --------
+    >>> from scipy.linalg import block_diag
     >>> A = [[1, 0],
     ...      [0, 1]]
     >>> B = [[3, 4, 5],
@@ -504,9 +528,9 @@ def block_diag(*arrs):
     bad_args = [k for k in range(len(arrs)) if arrs[k].ndim > 2]
     if bad_args:
         raise ValueError("arguments in the following positions have dimension "
-                            "greater than 2: %s" % bad_args)
+                         "greater than 2: %s" % bad_args)
 
-    shapes = np.array([a.shape for a in arrs])
+    shapes = np.array([a.shape if a.size > 0 else [0, 0] for a in arrs])
     out = np.zeros(np.sum(shapes, axis=0), dtype=arrs[0].dtype)
 
     r, c = 0, 0
@@ -516,6 +540,7 @@ def block_diag(*arrs):
         c += cc
     return out
 
+
 def companion(a):
     """
     Create a companion matrix.
@@ -525,15 +550,14 @@ def companion(a):
 
     Parameters
     ----------
-    a : array_like
+    a : (N,) array_like
         1-D array of polynomial coefficients.  The length of `a` must be
         at least two, and ``a[0]`` must not be zero.
 
     Returns
     -------
-    c : ndarray
-        A square array of shape ``(n-1, n-1)``, where `n` is the length
-        of `a`.  The first row of `c` is ``-a[1:]/a[0]``, and the first
+    c : (N-1, N-1) ndarray
+        The first row of `c` is ``-a[1:]/a[0]``, and the first
         sub-diagonal is all ones.  The data-type of the array is the same
         as the data-type of ``1.0*a[0]``.
 
@@ -564,7 +588,8 @@ def companion(a):
     a = np.atleast_1d(a)
 
     if a.ndim != 1:
-        raise ValueError("Incorrect shape for `a`.  `a` must be one-dimensional.")
+        raise ValueError("Incorrect shape for `a`.  `a` must be "
+                         "one-dimensional.")
 
     if a.size < 2:
         raise ValueError("The length of `a` must be at least 2.")
@@ -572,16 +597,62 @@ def companion(a):
     if a[0] == 0:
         raise ValueError("The first coefficient in `a` must not be zero.")
 
-    first_row = -a[1:]/(1.0*a[0])
+    first_row = -a[1:] / (1.0 * a[0])
     n = a.size
-    c = np.zeros((n-1, n-1), dtype=first_row.dtype)
+    c = np.zeros((n - 1, n - 1), dtype=first_row.dtype)
     c[0] = first_row
-    c[range(1,n-1), range(0, n-2)] = 1
+    c[list(range(1, n - 1)), list(range(0, n - 2))] = 1
     return c
 
 
+def helmert(n, full=False):
+    """
+    Create a Helmert matrix of order `n`.
+
+    This has applications in statistics, compositional or simplicial analysis,
+    and in Aitchison geometry.
+
+    Parameters
+    ----------
+    n : int
+        The size of the array to create.
+    full : bool, optional
+        If True the (n, n) ndarray will be returned.
+        Otherwise the submatrix that does not include the first
+        row will be returned.
+        Default: False.
+
+    Returns
+    -------
+    M : ndarray
+        The Helmert matrix.
+        The shape is (n, n) or (n-1, n) depending on the `full` argument.
+
+    Examples
+    --------
+    >>> from scipy.linalg import helmert
+    >>> helmert(5, full=True)
+    array([[ 0.4472136 ,  0.4472136 ,  0.4472136 ,  0.4472136 ,  0.4472136 ],
+           [ 0.70710678, -0.70710678,  0.        ,  0.        ,  0.        ],
+           [ 0.40824829,  0.40824829, -0.81649658,  0.        ,  0.        ],
+           [ 0.28867513,  0.28867513,  0.28867513, -0.8660254 ,  0.        ],
+           [ 0.2236068 ,  0.2236068 ,  0.2236068 ,  0.2236068 , -0.89442719]])
+
+    """
+    H = np.tril(np.ones((n, n)), -1) - np.diag(np.arange(n))
+    d = np.arange(n) * np.arange(1, n+1)
+    H[0] = 1
+    d[0] = n
+    H_full = H / np.sqrt(d)[:, np.newaxis]
+    if full:
+        return H_full
+    else:
+        return H_full[1:]
+
+
 def hilbert(n):
-    """Create a Hilbert matrix of order n.
+    """
+    Create a Hilbert matrix of order `n`.
 
     Returns the `n` by `n` array with entries `h[i,j] = 1 / (i + j + 1)`.
 
@@ -592,8 +663,12 @@ def hilbert(n):
 
     Returns
     -------
-    h : ndarray with shape (n, n)
-        The Hilber matrix.
+    h : (n, n) ndarray
+        The Hilbert matrix.
+
+    See Also
+    --------
+    invhilbert : Compute the inverse of a Hilbert matrix.
 
     Notes
     -----
@@ -601,6 +676,7 @@ def hilbert(n):
 
     Examples
     --------
+    >>> from scipy.linalg import hilbert
     >>> hilbert(3)
     array([[ 1.        ,  0.5       ,  0.33333333],
            [ 0.5       ,  0.33333333,  0.25      ],
@@ -608,39 +684,50 @@ def hilbert(n):
 
     """
     values = 1.0 / (1.0 + np.arange(2 * n - 1))
-    h = hankel(values[:n], r=values[n-1:])
+    h = hankel(values[:n], r=values[n - 1:])
     return h
 
 
 def invhilbert(n, exact=False):
-    """Compute the inverse of the Hilbert matrix of order `n`.
+    """
+    Compute the inverse of the Hilbert matrix of order `n`.
+
+    The entries in the inverse of a Hilbert matrix are integers.  When `n`
+    is greater than 14, some entries in the inverse exceed the upper limit
+    of 64 bit integers.  The `exact` argument provides two options for
+    dealing with these large integers.
 
     Parameters
     ----------
     n : int
         The order of the Hilbert matrix.
-    exact : bool
-        If False, the data type of the array that is returned in np.float64,
+    exact : bool, optional
+        If False, the data type of the array that is returned is np.float64,
         and the array is an approximation of the inverse.
-        If True, the array is exact integer array.  To represent the exact
-        inverse when n > 14, the returned array is an object array of long
-        integers.  For n <= 14, the exact inverse is returned as an array
-        with data type np.int64.
+        If True, the array is the exact integer inverse array.  To represent
+        the exact inverse when n > 14, the returned array is an object array
+        of long integers.  For n <= 14, the exact inverse is returned as an
+        array with data type np.int64.
 
     Returns
     -------
-    invh : ndarray with shape (n, n)
-        The data type of the array is np.float64 is exact is False.
-        If exact is True, the data type is either np.int64 (for n <= 14)
+    invh : (n, n) ndarray
+        The data type of the array is np.float64 if `exact` is False.
+        If `exact` is True, the data type is either np.int64 (for n <= 14)
         or object (for n > 14).  In the latter case, the objects in the
         array will be long integers.
 
+    See Also
+    --------
+    hilbert : Create a Hilbert matrix.
+
     Notes
     -----
     .. versionadded:: 0.10.0
 
     Examples
     --------
+    >>> from scipy.linalg import invhilbert
     >>> invhilbert(4)
     array([[   16.,  -120.,   240.,  -140.],
            [ -120.,  1200., -2700.,  1680.],
@@ -655,7 +742,9 @@ def invhilbert(n, exact=False):
     4.2475099528537506e+19
     >>> invhilbert(16, exact=True)[7,7]
     42475099528537378560L
+
     """
+    from scipy.special import comb
     if exact:
         if n > 14:
             dtype = object
@@ -667,10 +756,272 @@ def invhilbert(n, exact=False):
     for i in xrange(n):
         for j in xrange(0, i + 1):
             s = i + j
-            invh[i, j] = ((-1)**s * (s + 1) *
+            invh[i, j] = ((-1) ** s * (s + 1) *
                           comb(n + i, n - j - 1, exact) *
                           comb(n + j, n - i - 1, exact) *
                           comb(s, i, exact) ** 2)
             if i != j:
                 invh[j, i] = invh[i, j]
     return invh
+
+
+def pascal(n, kind='symmetric', exact=True):
+    """
+    Returns the n x n Pascal matrix.
+
+    The Pascal matrix is a matrix containing the binomial coefficients as
+    its elements.
+
+    Parameters
+    ----------
+    n : int
+        The size of the matrix to create; that is, the result is an n x n
+        matrix.
+    kind : str, optional
+        Must be one of 'symmetric', 'lower', or 'upper'.
+        Default is 'symmetric'.
+    exact : bool, optional
+        If `exact` is True, the result is either an array of type
+        numpy.uint64 (if n < 35) or an object array of Python long integers.
+        If `exact` is False, the coefficients in the matrix are computed using
+        `scipy.special.comb` with `exact=False`.  The result will be a floating
+        point array, and the values in the array will not be the exact
+        coefficients, but this version is much faster than `exact=True`.
+
+    Returns
+    -------
+    p : (n, n) ndarray
+        The Pascal matrix.
+
+    See Also
+    --------
+    invpascal
+
+    Notes
+    -----
+    See http://en.wikipedia.org/wiki/Pascal_matrix for more information
+    about Pascal matrices.
+
+    .. versionadded:: 0.11.0
+
+    Examples
+    --------
+    >>> from scipy.linalg import pascal
+    >>> pascal(4)
+    array([[ 1,  1,  1,  1],
+           [ 1,  2,  3,  4],
+           [ 1,  3,  6, 10],
+           [ 1,  4, 10, 20]], dtype=uint64)
+    >>> pascal(4, kind='lower')
+    array([[1, 0, 0, 0],
+           [1, 1, 0, 0],
+           [1, 2, 1, 0],
+           [1, 3, 3, 1]], dtype=uint64)
+    >>> pascal(50)[-1, -1]
+    25477612258980856902730428600L
+    >>> from scipy.special import comb
+    >>> comb(98, 49, exact=True)
+    25477612258980856902730428600L
+
+    """
+
+    from scipy.special import comb
+    if kind not in ['symmetric', 'lower', 'upper']:
+        raise ValueError("kind must be 'symmetric', 'lower', or 'upper'")
+
+    if exact:
+        if n >= 35:
+            L_n = np.empty((n, n), dtype=object)
+            L_n.fill(0)
+        else:
+            L_n = np.zeros((n, n), dtype=np.uint64)
+        for i in range(n):
+            for j in range(i + 1):
+                L_n[i, j] = comb(i, j, exact=True)
+    else:
+        L_n = comb(*np.ogrid[:n, :n])
+
+    if kind is 'lower':
+        p = L_n
+    elif kind is 'upper':
+        p = L_n.T
+    else:
+        p = np.dot(L_n, L_n.T)
+
+    return p
+
+
+def invpascal(n, kind='symmetric', exact=True):
+    """
+    Returns the inverse of the n x n Pascal matrix.
+
+    The Pascal matrix is a matrix containing the binomial coefficients as
+    its elements.
+
+    Parameters
+    ----------
+    n : int
+        The size of the matrix to create; that is, the result is an n x n
+        matrix.
+    kind : str, optional
+        Must be one of 'symmetric', 'lower', or 'upper'.
+        Default is 'symmetric'.
+    exact : bool, optional
+        If `exact` is True, the result is either an array of type
+        `numpy.int64` (if `n` <= 35) or an object array of Python integers.
+        If `exact` is False, the coefficients in the matrix are computed using
+        `scipy.special.comb` with `exact=False`.  The result will be a floating
+        point array, and for large `n`, the values in the array will not be the
+        exact coefficients.
+
+    Returns
+    -------
+    invp : (n, n) ndarray
+        The inverse of the Pascal matrix.
+
+    See Also
+    --------
+    pascal
+
+    Notes
+    -----
+
+    .. versionadded:: 0.16.0
+
+    References
+    ----------
+    .. [1] "Pascal matrix",  http://en.wikipedia.org/wiki/Pascal_matrix
+    .. [2] Cohen, A. M., "The inverse of a Pascal matrix", Mathematical
+           Gazette, 59(408), pp. 111-112, 1975.
+
+    Examples
+    --------
+    >>> from scipy.linalg import invpascal, pascal
+    >>> invp = invpascal(5)
+    >>> invp
+    array([[  5, -10,  10,  -5,   1],
+           [-10,  30, -35,  19,  -4],
+           [ 10, -35,  46, -27,   6],
+           [ -5,  19, -27,  17,  -4],
+           [  1,  -4,   6,  -4,   1]])
+
+    >>> p = pascal(5)
+    >>> p.dot(invp)
+    array([[ 1.,  0.,  0.,  0.,  0.],
+           [ 0.,  1.,  0.,  0.,  0.],
+           [ 0.,  0.,  1.,  0.,  0.],
+           [ 0.,  0.,  0.,  1.,  0.],
+           [ 0.,  0.,  0.,  0.,  1.]])
+
+    An example of the use of `kind` and `exact`:
+
+    >>> invpascal(5, kind='lower', exact=False)
+    array([[ 1., -0.,  0., -0.,  0.],
+           [-1.,  1., -0.,  0., -0.],
+           [ 1., -2.,  1., -0.,  0.],
+           [-1.,  3., -3.,  1., -0.],
+           [ 1., -4.,  6., -4.,  1.]])
+
+    """
+    from scipy.special import comb
+
+    if kind not in ['symmetric', 'lower', 'upper']:
+        raise ValueError("'kind' must be 'symmetric', 'lower' or 'upper'.")
+
+    if kind == 'symmetric':
+        if exact:
+            if n > 34:
+                dt = object
+            else:
+                dt = np.int64
+        else:
+            dt = np.float64
+        invp = np.empty((n, n), dtype=dt)
+        for i in range(n):
+            for j in range(0, i + 1):
+                v = 0
+                for k in range(n - i):
+                    v += comb(i + k, k, exact=exact) * comb(i + k, i + k - j,
+                                                            exact=exact)
+                invp[i, j] = (-1)**(i - j) * v
+                if i != j:
+                    invp[j, i] = invp[i, j]
+    else:
+        # For the 'lower' and 'upper' cases, we computer the inverse by
+        # changing the sign of every other diagonal of the pascal matrix.
+        invp = pascal(n, kind=kind, exact=exact)
+        if invp.dtype == np.uint64:
+            # This cast from np.uint64 to int64 OK, because if `kind` is not
+            # "symmetric", the values in invp are all much less than 2**63.
+            invp = invp.view(np.int64)
+
+        # The toeplitz matrix has alternating bands of 1 and -1.
+        invp *= toeplitz((-1)**np.arange(n)).astype(invp.dtype)
+
+    return invp
+
+
+def dft(n, scale=None):
+    """
+    Discrete Fourier transform matrix.
+
+    Create the matrix that computes the discrete Fourier transform of a
+    sequence [1]_.  The n-th primitive root of unity used to generate the
+    matrix is exp(-2*pi*i/n), where i = sqrt(-1).
+
+    Parameters
+    ----------
+    n : int
+        Size the matrix to create.
+    scale : str, optional
+        Must be None, 'sqrtn', or 'n'.
+        If `scale` is 'sqrtn', the matrix is divided by `sqrt(n)`.
+        If `scale` is 'n', the matrix is divided by `n`.
+        If `scale` is None (the default), the matrix is not normalized, and the
+        return value is simply the Vandermonde matrix of the roots of unity.
+
+    Returns
+    -------
+    m : (n, n) ndarray
+        The DFT matrix.
+
+    Notes
+    -----
+    When `scale` is None, multiplying a vector by the matrix returned by
+    `dft` is mathematically equivalent to (but much less efficient than)
+    the calculation performed by `scipy.fftpack.fft`.
+
+    .. versionadded:: 0.14.0
+
+    References
+    ----------
+    .. [1] "DFT matrix", http://en.wikipedia.org/wiki/DFT_matrix
+
+    Examples
+    --------
+    >>> from scipy.linalg import dft
+    >>> np.set_printoptions(precision=5, suppress=True)
+    >>> x = np.array([1, 2, 3, 0, 3, 2, 1, 0])
+    >>> m = dft(8)
+    >>> m.dot(x)   # Compute the DFT of x
+    array([ 12.+0.j,  -2.-2.j,   0.-4.j,  -2.+2.j,   4.+0.j,  -2.-2.j,
+            -0.+4.j,  -2.+2.j])
+
+    Verify that ``m.dot(x)`` is the same as ``fft(x)``.
+
+    >>> from scipy.fftpack import fft
+    >>> fft(x)     # Same result as m.dot(x)
+    array([ 12.+0.j,  -2.-2.j,   0.-4.j,  -2.+2.j,   4.+0.j,  -2.-2.j,
+             0.+4.j,  -2.+2.j])
+    """
+    if scale not in [None, 'sqrtn', 'n']:
+        raise ValueError("scale must be None, 'sqrtn', or 'n'; "
+                         "%r is not valid." % (scale,))
+
+    omegas = np.exp(-2j * np.pi * np.arange(n) / n).reshape(-1, 1)
+    m = omegas ** np.arange(n)
+    if scale == 'sqrtn':
+        m /= math.sqrt(n)
+    elif scale == 'n':
+        m /= n
+    return m
