@@ -1,9 +1,12 @@
+from __future__ import division, print_function, absolute_import
+
 import numpy as np
 from numpy.dual import eig
-from scipy.misc import comb
+from scipy.special import comb
 from scipy import linspace, pi, exp
+from scipy.signal import convolve
 
-__all__ = ['daub', 'qmf', 'cascade', 'morlet']
+__all__ = ['daub', 'qmf', 'cascade', 'morlet', 'ricker', 'cwt']
 
 
 def daub(p):
@@ -18,6 +21,11 @@ def daub(p):
     p : int
         Order of the zero at f=1/2, can have values from 1 to 34.
 
+    Returns
+    -------
+    daub : ndarray
+        Return
+
     """
     sqrt = np.sqrt
     if p < 1:
@@ -45,12 +53,12 @@ def daub(p):
             P = [comb(p - 1 + k, k, exact=1) for k in range(p)][::-1]
             yj = np.roots(P)
         else:  # try different polynomial --- needs work
-            P = [comb(p - 1 + k, k, exact=1) / 4.0 ** k
+            P = [comb(p - 1 + k, k, exact=1) / 4.0**k
                  for k in range(p)][::-1]
             yj = np.roots(P) / 4
         # for each root, compute two z roots, select the one with |z|>1
         # Build up final polynomial
-        c = np.poly1d([1, 1]) ** p
+        c = np.poly1d([1, 1])**p
         q = np.poly1d([1])
         for k in range(p - 1):
             yval = yj[k]
@@ -67,52 +75,46 @@ def daub(p):
         return q.c[::-1]
     else:
         raise ValueError("Polynomial factorization does not work "
-              "well for p too large.")
+                         "well for p too large.")
 
 
 def qmf(hk):
-    """Return high-pass qmf filter from low-pass
+    """
+    Return high-pass qmf filter from low-pass
+
+    Parameters
+    ----------
+    hk : array_like
+        Coefficients of high-pass filter.
+
     """
     N = len(hk) - 1
     asgn = [{0: 1, 1: -1}[k % 2] for k in range(N + 1)]
     return hk[::-1] * np.array(asgn)
 
 
-def wavedec(amn, hk):
-    gk = qmf(hk)
-    return NotImplemented
-
-
 def cascade(hk, J=7):
     """
-    Return (x, phi, psi) at dyadic points K/2**J from filter coefficients.
+    Return (x, phi, psi) at dyadic points ``K/2**J`` from filter coefficients.
 
     Parameters
     ----------
-    hk :
+    hk : array_like
         Coefficients of low-pass filter.
-    J : int. optional
-        Values will be computed at grid points ``K/2**J``.
+    J : int, optional
+        Values will be computed at grid points ``K/2**J``. Default is 7.
 
     Returns
     -------
-    x :
-        The dyadic points K/2**J for ``K=0...N * (2**J)-1`` where
-        ``len(hk) = len(gk) = N+1``
-    phi :
+    x : ndarray
+        The dyadic points ``K/2**J`` for ``K=0...N * (2**J)-1`` where
+        ``len(hk) = len(gk) = N+1``.
+    phi : ndarray
         The scaling function ``phi(x)`` at `x`:
-
-                     N
-          phi(x) = sum   hk * phi(2x-k)
-                     k=0
-
-    psi :
+        ``phi(x) = sum(hk * phi(2x-k))``, where k is from 0 to N.
+    psi : ndarray, optional
         The wavelet function ``psi(x)`` at `x`:
-
-                     N
-          phi(x) = sum   gk * phi(2x-k)
-                     k=0
-
+        ``phi(x) = sum(gk * phi(2x-k))``, where k is from 0 to N.
         `psi` is only returned if `gk` is not None.
 
     Notes
@@ -123,7 +125,6 @@ def cascade(hk, J=7):
     end.
 
     """
-
     N = len(hk) - 1
 
     if (J > 30 - np.log2(N + 1)):
@@ -207,42 +208,154 @@ def morlet(M, w=5.0, s=1.0, complete=True):
     ----------
     M : int
         Length of the wavelet.
-    w : float
-        Omega0
-    s : float
-        Scaling factor, windowed from -s*2*pi to +s*2*pi.
-    complete : bool
+    w : float, optional
+        Omega0. Default is 5
+    s : float, optional
+        Scaling factor, windowed from ``-s*2*pi`` to ``+s*2*pi``. Default is 1.
+    complete : bool, optional
         Whether to use the complete or the standard version.
 
+    Returns
+    -------
+    morlet : (M,) ndarray
+
+    See Also
+    --------
+    scipy.signal.gausspulse
+
     Notes
     -----
-    The standard version:
+    The standard version::
+
         pi**-0.25 * exp(1j*w*x) * exp(-0.5*(x**2))
 
-        This commonly used wavelet is often referred to simply as the
-        Morlet wavelet.  Note that, this simplified version can cause
-        admissibility problems at low values of w.
+    This commonly used wavelet is often referred to simply as the
+    Morlet wavelet.  Note that this simplified version can cause
+    admissibility problems at low values of w.
+
+    The complete version::
 
-    The complete version:
         pi**-0.25 * (exp(1j*w*x) - exp(-0.5*(w**2))) * exp(-0.5*(x**2))
 
-        The complete version of the Morlet wavelet, with a correction
-        term to improve admissibility. For w greater than 5, the
-        correction term is negligible.
+    The complete version of the Morlet wavelet, with a correction
+    term to improve admissibility. For w greater than 5, the
+    correction term is negligible.
 
     Note that the energy of the return wavelet is not normalised
     according to s.
 
     The fundamental frequency of this wavelet in Hz is given
-    by f = 2*s*w*r / M where r is the sampling rate.
+    by ``f = 2*s*w*r / M`` where r is the sampling rate.
 
     """
     x = linspace(-s * 2 * pi, s * 2 * pi, M)
     output = exp(1j * w * x)
 
     if complete:
-        output -= exp(-0.5 * (w ** 2))
+        output -= exp(-0.5 * (w**2))
 
-    output *= exp(-0.5 * (x ** 2)) * pi ** (-0.25)
+    output *= exp(-0.5 * (x**2)) * pi**(-0.25)
 
     return output
+
+
+def ricker(points, a):
+    """
+    Return a Ricker wavelet, also known as the "Mexican hat wavelet".
+
+    It models the function:
+
+        ``A (1 - x^2/a^2) exp(-x^2/2 a^2)``,
+
+    where ``A = 2/sqrt(3a)pi^1/4``.
+
+    Parameters
+    ----------
+    points : int
+        Number of points in `vector`.
+        Will be centered around 0.
+    a : scalar
+        Width parameter of the wavelet.
+
+    Returns
+    -------
+    vector : (N,) ndarray
+        Array of length `points` in shape of ricker curve.
+
+    Examples
+    --------
+    >>> from scipy import signal
+    >>> import matplotlib.pyplot as plt
+
+    >>> points = 100
+    >>> a = 4.0
+    >>> vec2 = signal.ricker(points, a)
+    >>> print(len(vec2))
+    100
+    >>> plt.plot(vec2)
+    >>> plt.show()
+
+    """
+    A = 2 / (np.sqrt(3 * a) * (np.pi**0.25))
+    wsq = a**2
+    vec = np.arange(0, points) - (points - 1.0) / 2
+    xsq = vec**2
+    mod = (1 - xsq / wsq)
+    gauss = np.exp(-xsq / (2 * wsq))
+    total = A * mod * gauss
+    return total
+
+
+def cwt(data, wavelet, widths):
+    """
+    Continuous wavelet transform.
+
+    Performs a continuous wavelet transform on `data`,
+    using the `wavelet` function. A CWT performs a convolution
+    with `data` using the `wavelet` function, which is characterized
+    by a width parameter and length parameter.
+
+    Parameters
+    ----------
+    data : (N,) ndarray
+        data on which to perform the transform.
+    wavelet : function
+        Wavelet function, which should take 2 arguments.
+        The first argument is the number of points that the returned vector
+        will have (len(wavelet(width,length)) == length).
+        The second is a width parameter, defining the size of the wavelet
+        (e.g. standard deviation of a gaussian). See `ricker`, which
+        satisfies these requirements.
+    widths : (M,) sequence
+        Widths to use for transform.
+
+    Returns
+    -------
+    cwt: (M, N) ndarray
+        Will have shape of (len(widths), len(data)).
+
+    Notes
+    -----
+    >>> length = min(10 * width[ii], len(data))
+    >>> cwt[ii,:] = scipy.signal.convolve(data, wavelet(length,
+    ...                                       width[ii]), mode='same')
+
+    Examples
+    --------
+    >>> from scipy import signal
+    >>> import matplotlib.pyplot as plt
+    >>> t = np.linspace(-1, 1, 200, endpoint=False)
+    >>> sig  = np.cos(2 * np.pi * 7 * t) + signal.gausspulse(t - 0.4, fc=2)
+    >>> widths = np.arange(1, 31)
+    >>> cwtmatr = signal.cwt(sig, signal.ricker, widths)
+    >>> plt.imshow(cwtmatr, extent=[-1, 1, 1, 31], cmap='PRGn', aspect='auto',
+    ...            vmax=abs(cwtmatr).max(), vmin=-abs(cwtmatr).max())
+    >>> plt.show()
+
+    """
+    output = np.zeros([len(widths), len(data)])
+    for ind, width in enumerate(widths):
+        wavelet_data = wavelet(min(10 * width, len(data)), width)
+        output[ind, :] = convolve(data, wavelet_data,
+                                  mode='same')
+    return output
