@@ -26,6 +26,7 @@ Example:
    :toctree: generated/
 
    errprint
+   SpecialFunctionWarning -- Warning that can be issued with ``errprint(True)``
 
 Available functions
 ===================
@@ -40,7 +41,8 @@ Airy functions
    airye    -- Exponentially scaled Airy functions
    ai_zeros -- [+]Zeros of Airy functions Ai(x) and Ai'(x)
    bi_zeros -- [+]Zeros of Airy functions Bi(x) and Bi'(x)
-
+   itairy   --
+   
 
 Elliptic Functions and Integrals
 --------------------------------
@@ -61,7 +63,6 @@ Bessel Functions
 .. autosummary::
    :toctree: generated/
 
-   jn       -- Bessel function of integer order and real argument.
    jv       -- Bessel function of real-valued order and complex argument.
    jve      -- Exponentially scaled Bessel function.
    yn       -- Bessel function of second kind (integer order).
@@ -196,8 +197,12 @@ Raw Statistical Functions
    bdtr       -- Sum of terms 0 through k of the binomial pdf.
    bdtrc      -- Sum of terms k+1 through n of the binomial pdf.
    bdtri      -- Inverse of bdtr
+   bdtrik     --
+   bdtrin     --
    btdtr      -- Integral from 0 to x of beta pdf.
    btdtri     -- Quantiles of beta distribution
+   btdtria    --
+   btdtrib    --
    fdtr       -- Integral from 0 to x of F pdf.
    fdtrc      -- Integral from x to infinity under F pdf.
    fdtri      -- Inverse of fdtrc
@@ -209,17 +214,37 @@ Raw Statistical Functions
    nbdtr      -- Sum of terms 0 through k of the negative binomial pdf.
    nbdtrc     -- Sum of terms k+1 to infinity under negative binomial pdf.
    nbdtri     -- Inverse of nbdtr
+   nbdtrik    --
+   nbdtrin    --
+   ncfdtr     -- CDF of non-central t distribution.
+   ncfdtridfd -- Find degrees of freedom (denominator) of noncentral F distribution.
+   ncfdtridfn -- Find degrees of freedom (numerator) of noncentral F distribution.
+   ncfdtri    -- Inverse CDF of noncentral F distribution.
+   ncfdtrinc  -- Find noncentrality parameter of noncentral F distribution.
+   nctdtr     -- CDF of noncentral t distribution.
+   nctdtridf  -- Find degrees of freedom of noncentral t distribution.
+   nctdtrit   -- Inverse CDF of noncentral t distribution.
+   nctdtrinc  -- Find noncentrality parameter of noncentral t distribution.
+   nrdtrimn   -- Find mean of normal distribution from cdf and std.
+   nrdtrisd   -- Find std of normal distribution from cdf and mean.
    pdtr       -- Sum of terms 0 through k of the Poisson pdf.
    pdtrc      -- Sum of terms k+1 to infinity of the Poisson pdf.
    pdtri      -- Inverse of pdtr
+   pdtrik     --
    stdtr      -- Integral from -infinity to t of the Student-t pdf.
    stdtridf   --
    stdtrit    --
    chdtr      -- Integral from 0 to x of the Chi-square pdf.
    chdtrc     -- Integral from x to infnity of Chi-square pdf.
    chdtri     -- Inverse of chdtrc.
+   chdtriv    --
    ndtr       -- Integral from -infinity to x of standard normal pdf
+   log_ndtr   -- Logarithm of integral from -infinity to x of standard normal pdf
    ndtri      -- Inverse of ndtr (quantiles)
+   chndtr     --
+   chndtridf  --
+   chndtrinc  --
+   chndtrix   --
    smirnov    -- Kolmogorov-Smirnov complementary CDF for one-sided test statistic (Dn+ or Dn-)
    smirnovi   -- Inverse of smirnov.
    kolmogorov -- The complementary CDF of the (scaled) two-sided test statistic (Kn*) valid for large n.
@@ -230,6 +255,20 @@ Raw Statistical Functions
    boxcox     -- Compute the Box-Cox transformation.
    boxcox1p   -- Compute the Box-Cox transformation.
 
+
+Information Theory Functions
+----------------------------
+
+.. autosummary::
+   :toctree: generated/
+
+   entr         -- entr(x) = -x*log(x)
+   rel_entr     -- rel_entr(x, y) = x*log(x/y)
+   kl_div       -- kl_div(x, y) = x*log(x/y) - x + y
+   huber        -- Huber loss function.
+   pseudo_huber -- Pseudo-Huber loss function.
+
+
 Gamma and Related Functions
 ---------------------------
 
@@ -250,7 +289,9 @@ Gamma and Related Functions
    psi          -- Logarithmic derivative of the gamma function.
    rgamma       -- One divided by the gamma function.
    polygamma    -- Nth derivative of psi function.
-   multigammaln
+   multigammaln -- Log of the multivariate gamma.
+   digamma      -- Digamma function (derivative of the logarithm of gamma).
+   poch         -- The Pochhammer symbol (rising factorial).
 
 
 Error Function and Fresnel Integrals
@@ -301,6 +342,16 @@ These are not universal functions:
    lpmn     -- [+]Associated Legendre Function of the first kind for real arguments.
    lqmn     -- [+]Associated Legendre Function of the second kind.
 
+Ellipsoidal Harmonics
+---------------------
+
+.. autosummary::
+   :toctree: generated/
+
+   ellip_harm    -- Ellipsoidal harmonic E
+   ellip_harm_2  -- Ellipsoidal harmonic F
+   ellip_normal  -- Ellipsoidal normalization constant
+
 Orthogonal polynomials
 ----------------------
 
@@ -309,6 +360,7 @@ The following functions evaluate values of orthogonal polynomials:
 .. autosummary::
    :toctree: generated/
 
+   assoc_laguerre
    eval_legendre
    eval_chebyt
    eval_chebyu
@@ -325,12 +377,14 @@ The following functions evaluate values of orthogonal polynomials:
    eval_sh_chebyu
    eval_sh_jacobi
 
-The functions below, in turn, return :ref:`orthopoly1d` objects, which
-functions similarly as :ref:`numpy.poly1d`.  The :ref:`orthopoly1d`
-class also has an attribute ``weights`` which returns the roots, weights,
-and total weights for the appropriate form of Gaussian quadrature.
-These are returned in an ``n x 3`` array with roots in the first column,
-weights in the second column, and total weights in the final column.
+The functions below, in turn, return the polynomial coefficients in
+:class:`~.orthopoly1d` objects, which function similarly as :ref:`numpy.poly1d`.
+The :class:`~.orthopoly1d` class also has an attribute ``weights`` which returns
+the roots, weights, and total weights for the appropriate form of Gaussian
+quadrature.  These are returned in an ``n x 3`` array with roots in the first
+column, weights in the second column, and total weights in the final column.
+Note that :class:`~.orthopoly1d` objects are converted to ``poly1d`` when doing
+arithmetic, and lose information of the original orthogonal polynomial.
 
 .. autosummary::
    :toctree: generated/
@@ -353,13 +407,10 @@ weights in the second column, and total weights in the final column.
 
 .. warning::
 
-   Large-order polynomials obtained from these functions
-   are numerically unstable.
+   Computing values of high-order polynomials (around ``order > 20``) using
+   polynomial coefficients is numerically unstable. To evaluate polynomial
+   values, the ``eval_*`` functions should be used instead.
 
-   ``orthopoly1d`` objects are converted to ``poly1d``, when doing
-   arithmetic.  ``numpy.poly1d`` works in power basis and cannot
-   represent high-order polynomials accurately, which can cause
-   significant inaccuracy.
 
 
 Hypergeometric Functions
@@ -500,7 +551,11 @@ Other Special Functions
 .. autosummary::
    :toctree: generated/
 
+   agm          -- Arithmetic-Geometric Mean
+   bernoulli    -- Bernoulli numbers
    binom        -- Binomial coefficient.
+   diric        -- Dirichlet function (periodic sinc)
+   euler        -- Euler numbers
    expn         -- Exponential integral.
    exp1         -- Exponential integral of order 1 (for complex argument)
    expi         -- Another exponential integral -- Ei(x)
@@ -550,8 +605,10 @@ from . import specfun
 from . import orthogonal
 from .orthogonal import *
 from .spfun_stats import multigammaln
+from ._ellip_harm import ellip_harm, ellip_harm_2, ellip_normal
 from .lambertw import lambertw
 
+
 __all__ = [s for s in dir() if not s.startswith('_')]
 
 from numpy.dual import register_func
